@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2016	PgPool Global Development Group
+ * Copyright (c) 2003-2026	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -139,14 +139,14 @@ static TSRel *
 relcache_lookup(TSRewriteContext * ctx)
 {
 #define ATTRDEFQUERY (Pgversion(ctx->backend)->major >= 73 ? \
-	"SELECT attname, pg_get_expr(d.adbin, d.adrelid), coalesce((pg_get_expr(d.adbin, d.adrelid) LIKE '%%now()%%' OR pg_get_expr(d.adbin, d.adrelid) LIKE '%%''now''::text%%' OR" \
-	" pg_get_expr(d.adbin, d.adrelid) LIKE '%%CURRENT_TIMESTAMP%%' OR pg_get_expr(d.adbin, d.adrelid) LIKE '%%CURRENT_TIME%%' OR pg_get_expr(d.adbin, d.adrelid) LIKE '%%CURRENT_DATE%%' OR" \
-	" pg_get_expr(d.adbin, d.adrelid) LIKE '%%LOCALTIME%%' OR pg_get_expr(d.adbin, d.adrelid) LIKE '%%LOCALTIMESTAMP%%')" \
-	" AND (a.atttypid = 'timestamp'::regtype::oid OR" \
-	" a.atttypid = 'timestamp with time zone'::regtype::oid OR" \
-	" a.atttypid = 'date'::regtype::oid OR" \
-	" a.atttypid = 'time'::regtype::oid OR" \
-	" a.atttypid = 'time with time zone'::regtype::oid)" \
+	"SELECT attname, pg_catalog.pg_get_expr(d.adbin, d.adrelid), coalesce((pg_catalog.pg_get_expr(d.adbin, d.adrelid) LIKE '%%now()%%' OR pg_catalog.pg_get_expr(d.adbin, d.adrelid) LIKE '%%''now''::text%%' OR" \
+	" pg_catalog.pg_get_expr(d.adbin, d.adrelid) LIKE '%%CURRENT_TIMESTAMP%%' OR pg_catalog.pg_get_expr(d.adbin, d.adrelid) LIKE '%%CURRENT_TIME%%' OR pg_catalog.pg_get_expr(d.adbin, d.adrelid) LIKE '%%CURRENT_DATE%%' OR" \
+	" pg_catalog.pg_get_expr(d.adbin, d.adrelid) LIKE '%%LOCALTIME%%' OR pg_catalog.pg_get_expr(d.adbin, d.adrelid) LIKE '%%LOCALTIMESTAMP%%')" \
+	" AND (a.atttypid = 'timestamp'::pg_catalog.regtype::pg_catalog.oid OR" \
+	" a.atttypid = 'timestamp with time zone'::pg_catalog.regtype::pg_catalog.oid OR" \
+	" a.atttypid = 'date'::pg_catalog.regtype::pg_catalog.oid OR" \
+	" a.atttypid = 'time'::pg_catalog.regtype::pg_catalog.oid OR" \
+	" a.atttypid = 'time with time zone'::pg_catalog.regtype::pg_catalog.oid)" \
     " , false)" \
 	" FROM pg_catalog.pg_class c, pg_catalog.pg_attribute a " \
 	" LEFT JOIN pg_catalog.pg_attrdef d ON (a.attrelid = d.adrelid AND a.attnum = d.adnum)" \
@@ -155,11 +155,11 @@ relcache_lookup(TSRewriteContext * ctx)
 	"SELECT attname, d.adsrc, coalesce((d.adsrc LIKE '%%now()%%' OR d.adsrc LIKE '%%''now''::text%%' OR" \
 	" d.adsrc LIKE '%%CURRENT_TIMESTAMP%%' OR d.adsrc LIKE '%%CURRENT_TIME%%' OR d.adsrc LIKE '%%CURRENT_DATE%%' OR" \
 	" d.adsrc LIKE '%%LOCALTIME%%' OR d.adsrc LIKE '%%LOCALTIMESTAMP%%')" \
-	" AND (a.atttypid = 'timestamp'::regtype::oid OR" \
-	" a.atttypid = 'timestamp with time zone'::regtype::oid OR" \
-	" a.atttypid = 'date'::regtype::oid OR" \
-	" a.atttypid = 'time'::regtype::oid OR" \
-	" a.atttypid = 'time with time zone'::regtype::oid)" \
+	" AND (a.atttypid = 'timestamp'::pg_catalog.regtype::pg_catalog.oid OR" \
+	" a.atttypid = 'timestamp with time zone'::pg_catalog.regtype::pg_catalog.oid OR" \
+	" a.atttypid = 'date'::pg_catalog.regtype::pg_catalog.oid OR" \
+	" a.atttypid = 'time'::pg_catalog.regtype::pg_catalog.oid OR" \
+	" a.atttypid = 'time with time zone'::pg_catalog.regtype::pg_catalog.oid)" \
     " , false)" \
 	" FROM pg_catalog.pg_class c, pg_catalog.pg_attribute a " \
 	" LEFT JOIN pg_catalog.pg_attrdef d ON (a.attrelid = d.adrelid AND a.attnum = d.adnum)" \
@@ -167,14 +167,14 @@ relcache_lookup(TSRewriteContext * ctx)
 	" ORDER BY a.attnum")
 
 #define ATTRDEFQUERY2 (Pgversion(ctx->backend)->major >= 73 ? \
-	"SELECT attname, pg_get_expr(d.adbin, d.adrelid), coalesce((pg_get_expr(d.adbin, d.adrelid) LIKE '%%now()%%' OR pg_get_expr(d.adbin, d.adrelid) LIKE '%%''now''::text%%' OR" \
-	" pg_get_expr(d.adbin, d.adrelid) LIKE '%%CURRENT_TIMESTAMP%%' OR pg_get_expr(d.adbin, d.adrelid) LIKE '%%CURRENT_TIME%%' OR pg_get_expr(d.adbin, d.adrelid) LIKE '%%CURRENT_DATE%%' OR" \
-	" pg_get_expr(d.adbin, d.adrelid) LIKE '%%LOCALTIME%%' OR pg_get_expr(d.adbin, d.adrelid) LIKE '%%LOCALTIMESTAMP%%')" \
-	" AND (a.atttypid = 'timestamp'::regtype::oid OR" \
-	" a.atttypid = 'timestamp with time zone'::regtype::oid OR" \
-	" a.atttypid = 'date'::regtype::oid OR" \
-	" a.atttypid = 'time'::regtype::oid OR" \
-	" a.atttypid = 'time with time zone'::regtype::oid)" \
+	"SELECT attname, pg_catalog.pg_get_expr(d.adbin, d.adrelid), coalesce((pg_catalog.pg_get_expr(d.adbin, d.adrelid) LIKE '%%now()%%' OR pg_catalog.pg_get_expr(d.adbin, d.adrelid) LIKE '%%''now''::text%%' OR" \
+	" pg_catalog.pg_get_expr(d.adbin, d.adrelid) LIKE '%%CURRENT_TIMESTAMP%%' OR pg_catalog.pg_get_expr(d.adbin, d.adrelid) LIKE '%%CURRENT_TIME%%' OR pg_catalog.pg_get_expr(d.adbin, d.adrelid) LIKE '%%CURRENT_DATE%%' OR" \
+	" pg_catalog.pg_get_expr(d.adbin, d.adrelid) LIKE '%%LOCALTIME%%' OR pg_catalog.pg_get_expr(d.adbin, d.adrelid) LIKE '%%LOCALTIMESTAMP%%')" \
+	" AND (a.atttypid = 'timestamp'::pg_catalog.regtype::pg_catalog.oid OR" \
+	" a.atttypid = 'timestamp with time zone'::pg_catalog.regtype::pg_catalog.oid OR" \
+	" a.atttypid = 'date'::pg_catalog.regtype::pg_catalog.oid OR" \
+	" a.atttypid = 'time'::pg_catalog.regtype::pg_catalog.oid OR" \
+	" a.atttypid = 'time with time zone'::pg_catalog.regtype::pg_catalog.oid)" \
     " , false)" \
 	" FROM pg_catalog.pg_class c, pg_catalog.pg_attribute a " \
 	" LEFT JOIN pg_catalog.pg_attrdef d ON (a.attrelid = d.adrelid AND a.attnum = d.adnum)" \
@@ -183,11 +183,11 @@ relcache_lookup(TSRewriteContext * ctx)
 	"SELECT attname, d.adsrc, coalesce((d.adsrc LIKE '%%now()%%' OR d.adsrc LIKE '%%''now''::text%%' OR" \
 	" d.adsrc LIKE '%%CURRENT_TIMESTAMP%%' OR d.adsrc LIKE '%%CURRENT_TIME%%' OR d.adsrc LIKE '%%CURRENT_DATE%%' OR" \
 	" d.adsrc LIKE '%%LOCALTIME%%' OR d.adsrc LIKE '%%LOCALTIMESTAMP%%')" \
-	" AND (a.atttypid = 'timestamp'::regtype::oid OR" \
-	" a.atttypid = 'timestamp with time zone'::regtype::oid OR" \
-	" a.atttypid = 'date'::regtype::oid OR" \
-	" a.atttypid = 'time'::regtype::oid OR" \
-	" a.atttypid = 'time with time zone'::regtype::oid)" \
+	" AND (a.atttypid = 'timestamp'::pg_catalog.regtype::pg_catalog.oid OR" \
+	" a.atttypid = 'timestamp with time zone'::pg_catalog.regtype::pg_catalog.oid OR" \
+	" a.atttypid = 'date'::pg_catalog.regtype::pg_catalog.oid OR" \
+	" a.atttypid = 'time'::pg_catalog.regtype::pg_catalog.oid OR" \
+	" a.atttypid = 'time with time zone'::pg_catalog.regtype::pg_catalog.oid)" \
     " , false)" \
 	" FROM pg_catalog.pg_class c, pg_catalog.pg_attribute a " \
 	" LEFT JOIN pg_catalog.pg_attrdef d ON (a.attrelid = d.adrelid AND a.attnum = d.adnum)" \
@@ -195,31 +195,31 @@ relcache_lookup(TSRewriteContext * ctx)
 	" ORDER BY a.attnum")
 
 #define ATTRDEFQUERY3 (Pgversion(ctx->backend)->major >= 73 ? \
-	"SELECT attname, pg_get_expr(d.adbin, d.adrelid), coalesce((pg_get_expr(d.adbin, d.adrelid) LIKE '%%now()%%' OR pg_get_expr(d.adbin, d.adrelid) LIKE '%%''now''::text%%' OR" \
-	" pg_get_expr(d.adbin, d.adrelid) LIKE '%%CURRENT_TIMESTAMP%%' OR pg_get_expr(d.adbin, d.adrelid) LIKE '%%CURRENT_TIME%%' OR pg_get_expr(d.adbin, d.adrelid) LIKE '%%CURRENT_DATE%%' OR" \
-	" pg_get_expr(d.adbin, d.adrelid) LIKE '%%LOCALTIME%%' OR pg_get_expr(d.adbin, d.adrelid) LIKE '%%LOCALTIMESTAMP%%')" \
-	" AND (a.atttypid = 'timestamp'::regtype::oid OR" \
-	" a.atttypid = 'timestamp with time zone'::regtype::oid OR" \
-	" a.atttypid = 'date'::regtype::oid OR" \
-	" a.atttypid = 'time'::regtype::oid OR" \
-	" a.atttypid = 'time with time zone'::regtype::oid)" \
+	"SELECT attname, pg_catalog.pg_get_expr(d.adbin, d.adrelid), coalesce((pg_catalog.pg_get_expr(d.adbin, d.adrelid) LIKE '%%now()%%' OR pg_catalog.pg_get_expr(d.adbin, d.adrelid) LIKE '%%''now''::text%%' OR" \
+	" pg_catalog.pg_get_expr(d.adbin, d.adrelid) LIKE '%%CURRENT_TIMESTAMP%%' OR pg_catalog.pg_get_expr(d.adbin, d.adrelid) LIKE '%%CURRENT_TIME%%' OR pg_catalog.pg_get_expr(d.adbin, d.adrelid) LIKE '%%CURRENT_DATE%%' OR" \
+	" pg_catalog.pg_get_expr(d.adbin, d.adrelid) LIKE '%%LOCALTIME%%' OR pg_catalog.pg_get_expr(d.adbin, d.adrelid) LIKE '%%LOCALTIMESTAMP%%')" \
+	" AND (a.atttypid = 'timestamp'::pg_catalog.regtype::pg_catalog.oid OR" \
+	" a.atttypid = 'timestamp with time zone'::pg_catalog.regtype::pg_catalog.oid OR" \
+	" a.atttypid = 'date'::pg_catalog.regtype::pg_catalog.oid OR" \
+	" a.atttypid = 'time'::pg_catalog.regtype::pg_catalog.oid OR" \
+	" a.atttypid = 'time with time zone'::pg_catalog.regtype::pg_catalog.oid)" \
     " , false)" \
 	" FROM pg_catalog.pg_class c, pg_catalog.pg_attribute a " \
 	" LEFT JOIN pg_catalog.pg_attrdef d ON (a.attrelid = d.adrelid AND a.attnum = d.adnum)" \
-	" WHERE c.oid = a.attrelid AND a.attnum >= 1 AND a.attisdropped = 'f' AND c.oid = to_regclass('%s')" \
+	" WHERE c.oid = a.attrelid AND a.attnum >= 1 AND a.attisdropped = 'f' AND c.oid = pg_catalog.to_regclass('%s')" \
 	" ORDER BY a.attnum" : \
 	"SELECT attname, d.adsrc, coalesce((d.adsrc LIKE '%%now()%%' OR d.adsrc LIKE '%%''now''::text%%' OR" \
 	" d.adsrc LIKE '%%CURRENT_TIMESTAMP%%' OR d.adsrc LIKE '%%CURRENT_TIME%%' OR d.adsrc LIKE '%%CURRENT_DATE%%' OR" \
 	" d.adsrc LIKE '%%LOCALTIME%%' OR d.adsrc LIKE '%%LOCALTIMESTAMP%%')" \
-	" AND (a.atttypid = 'timestamp'::regtype::oid OR" \
-	" a.atttypid = 'timestamp with time zone'::regtype::oid OR" \
-	" a.atttypid = 'date'::regtype::oid OR" \
-	" a.atttypid = 'time'::regtype::oid OR" \
-	" a.atttypid = 'time with time zone'::regtype::oid)" \
+	" AND (a.atttypid = 'timestamp'::pg_catalog.regtype::pg_catalog.oid OR" \
+	" a.atttypid = 'timestamp with time zone'::pg_catalog.regtype::pg_catalog.oid OR" \
+	" a.atttypid = 'date'::pg_catalog.regtype::pg_catalog.oid OR" \
+	" a.atttypid = 'time'::pg_catalog.regtype::pg_catalog.oid OR" \
+	" a.atttypid = 'time with time zone'::pg_catalog.regtype::pg_catalog.oid)" \
     " , false)" \
 	" FROM pg_catalog.pg_class c, pg_catalog.pg_attribute a " \
 	" LEFT JOIN pg_catalog.pg_attrdef d ON (a.attrelid = d.adrelid AND a.attnum = d.adnum)" \
-	" WHERE c.oid = a.attrelid AND a.attnum >= 1 AND a.attisdropped = 'f' AND c.oid = to_regclass('%s')" \
+	" WHERE c.oid = a.attrelid AND a.attnum >= 1 AND a.attisdropped = 'f' AND c.oid = pg_catalog.to_regclass('%s')" \
 	" ORDER BY a.attnum")
 
 	char	   *query;
@@ -256,7 +256,7 @@ relcache_lookup(TSRewriteContext * ctx)
 	return (TSRel *) pool_search_relcache(ts_relcache, ctx->backend, table_name);
 }
 
-/* If timestamp value is required then return ts_const. Otherwize make
+/* If timestamp value is required then return ts_const. Otherwise make
  * a new ParamRef, whose number member is 0 and is overwritten at last,
  * and add it into params list in context.
  */
@@ -488,7 +488,7 @@ get_current_timestamp(POOL_CONNECTION_POOL * backend)
 	POOL_SELECT_RESULT *res;
 	static char timestamp[64];
 
-	do_query(MAIN(backend), "SELECT now()", &res, MAJOR(backend));
+	do_query(MAIN(backend), "SELECT pg_catalog.now()", &res, MAJOR(backend));
 
 	if (res->numrows != 1)
 	{
@@ -782,7 +782,7 @@ rewrite_timestamp_update(UpdateStmt *u_stmt, TSRewriteContext * ctx)
 /*
  * Rewrite `now()' to timestamp literal.
  * If rewrite_to_params is false then, we rewrite `now()' to timestamp constant.
- * Otherwize, we rewrite `now()' to params and expand that at Bind message.
+ * Otherwise, we rewrite `now()' to params and expand that at Bind message.
  * returns query string as palloced string, or NULL if not to need rewrite.
  */
 char *
@@ -1174,7 +1174,7 @@ bind_rewrite_timestamp(POOL_CONNECTION_POOL * backend,
 				copy_to += sizeof(int16);
 			}
 		}
-		/* otherwize, copy the original format codes as they are*/
+		/* otherwise, copy the original format codes as they are*/
 		else
 		{
 			copy_len = num_formats * sizeof(int16);
@@ -1358,6 +1358,44 @@ makeTypeCastFromSvfOp(SQLValueFunctionOp op)
  * Currently, the node type coverage extends to SelectStmt and everything
  * that could appear under it, but not other statement types.
  */
+
+/*
+ * expression_tree_walker() is designed to support routines that traverse
+ * a tree in a read-only fashion (although it will also work for routines
+ * that modify nodes in-place but never add/delete/replace nodes).
+ * A walker routine should look like this:
+ *
+ * bool my_walker (Node *node, my_struct *context)
+ * {
+ *		if (node == NULL)
+ *			return false;
+ *		// check for nodes that special work is required for, eg:
+ *		if (IsA(node, Var))
+ *		{
+ *			... do special actions for Var nodes
+ *		}
+ *		else if (IsA(node, ...))
+ *		{
+ *			... do special actions for other node types
+ *		}
+ *		// for any node type not specially processed, do:
+ *		return expression_tree_walker(node, my_walker, context);
+ * }
+ *
+ * The "context" argument points to a struct that holds whatever context
+ * information the walker routine needs --- it can be used to return data
+ * gathered by the walker, too.  This argument is not touched by
+ * expression_tree_walker, but it is passed down to recursive sub-invocations
+ * of my_walker.  The tree walk is started from a setup routine that
+ * fills in the appropriate context struct, calls my_walker with the top-level
+ * node of the tree, and then examines the results.
+ *
+ * The walker routine should return "false" to continue the tree walk, or
+ * "true" to abort the walk and immediately return "true" to the top-level
+ * caller.  This can be used to short-circuit the traversal if the walker
+ * has found what it came for.  "false" is returned to the top-level caller
+ * iff no invocation of the walker returned "true".
+ */
 bool
 raw_expression_tree_walker(Node *node,
 						   bool (*walker) (),
